/*
   program : backproject.c

   $Source: /home/yee/C/RCS/colin.c,v $
   $Author: yee $
   $Date: 1999/02/10 16:37:41 $
   $Revision: 1.9 $

   This file contains the code implementing colinear fan beam backprojection.
   It is called from code in backproject.c

   Compile Options : none

   Runtime Parameters : none

   Assumptions : none known

   Bugs : none known

   To Do : nothing

   $Log: colin.c,v $
   Revision 1.9  1999/02/10 16:37:41  yee
   *** empty log message ***

   Revision 1.8  1999/01/01 04:24:46  yee
   *** empty log message ***

   Revision 1.7  1998/12/21 15:41:00  yee
   working parallel. broken angle/colin.

   Revision 1.6  1998/08/27 22:54:08  yee
   working for parallel and colinear fan geometries

   Revision 1.1  1997/11/09 20:57:33  yee
   eliminated DetermineRay().

   Copyright 1997 by Conway Yee.  All rights reserved.
 */

char colin[] = "$Id: colin.c,v 1.9 1999/02/10 16:37:41 yee Exp $"; 

#include <math.h>

#include "codes.h"
#include "style.h"
#include "recon.h"
#include "ImageHeader.h"
#include "backproject.h"
 
/*****************************************************************************/
/*****************************************************************************/

/* this function weights the backprojection filter for co-linear fan
   beam data.  It is called by the generic filter code in filter.c */

#ifdef RECON_COLIN_FAN
PUBLIC double ColinBackprojectionFilterWeight
(
 int ndim,           /* dimensionality of data array */
 int i,              /* element number */
 void *free          /* degrees of freedom */
)
{ /* ColinProjectionFilterWeight() */
  return(0.5);
} /* ColinProjectionFilterWeight() */
#endif RECON_COLIN_FAN

/*****************************************************************************/

/* this function weights the projections for co-linear fan beam data */

#ifdef RECON_COLIN_FAN
PUBLIC double ColinProjectionWeight
(
 int ndim,           /* dimensionality of data array */
 int i,              /* element number */
 double *FloatFree   /* degrees of freedom */
)
{ /* ColinProjectionWeight() */
  double D=FloatFree[0];   /* FloatFree[0] = D, distance between focal point
                                             and center of rotation */
  double tau=FloatFree[1]; /* FloatFree[1] = tau, distance between elements */
  double loc;              /* location along array */

  loc = i*tau;

  return(D / SQRT(D*D+loc*loc));
} /* ColinProjectionWeight() */
#endif RECON_COLIN__FAN

/*****************************************************************************/

/* this function calculates the point from which a pixel is backprojected
   in units of distance for co-linear fan beam data. */

#ifdef RECON_COLIN_FAN
PUBLIC double ColinBackprojectPoint
(
 double x,          /* X position in 2D image space */
 double y,          /* Y position in 2D image space */
 double *qtau,      /* spacing of projection elements */
 double *ftau,      /* spacing of reconstructed image elements */
 double sin_t,      /* sin of angle made by projection with +X axis */
 double cos_t,      /* cos of angle made by projection with +X axis */
 double D           /* distance between origin and focal point */
)
{ /* ColinBackprojectPoint() */
  return(D*(x*sin_t-y*cos_t)/(D-(y*sin_t+x*cos_t)));  /* dist along detector */
} /* ColinBackprojectPoint() */
#endif RECON_COLIN_FAN

/*****************************************************************************/

/* this function is used by BackprojectProjection() for equi-colinear fan beam
   data to calculate the squared ratio of the distance between the origin to
   the focal point to the distance between the focal point to the projection
   of (x,y) onto the central ray. */

#ifdef RECON_COLIN_FAN
PUBLIC double U2
(
 double D,      /* distance between center of rotation and focal point */
 double x,      /* X coordinate of point */
 double y,      /* Y coordinate of point */
 double sin_t,  /* sin(angle of center ray of projection) */
 double cos_t   /* cos(angle of center ray of projection) */
)
{ /* U2() */
  double r=SQRT(x*x+y*y);                 /* distance of (x,y) */
  double sin_p= y/r;                      /* sin(phi) */
  double cos_p= x/r;                      /* cos(phi) */
  double sin_tp;                          /* sin(theta-phi) */
                                    /* Note: may need sin(theta+PI_2-phi) */

  if(fabs(r)<=EPSILON) {
    sin_p = 0.0;
    cos_p = 1.0;
  }

  sin_tp=sin_t*cos_p-cos_t*sin_p;    /* sin(theta-phi) */

  return(pow(((D+r*sin_tp)/D),2.0));
} /* U2() */
#endif RECON_COLIN_FAN
